/************************************************************************/
/*									*/
/*			Access Dallas 1-Wire Devices			*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/
#include "main.h"


#ifndef W1_OUT
//#define W1_PIN	PD6
#define W1_IN	PIND
#define W1_OUT	PORTD
#define W1_DDR	DDRD
#endif


bit w1_reset(uchar W1PIN)
{
  bit err;
  W1_OUT &= ~(1<<W1PIN);
  W1_DDR |= 1<<W1PIN;
  DELAY( DELAY_US( 480 ));			// 480 us
  cli();
  W1_DDR &= ~(1<<W1PIN);
  DELAY( DELAY_US( 66 ));
  err = W1_IN & (1<<W1PIN);			// no presence detect
  sei();
  DELAY( DELAY_US( 480 - 66 ));
  if( (W1_IN & (1<<W1PIN)) == 0 )		// short circuit
    err = 1;
  return err;
}


uchar w1_bit_io( bit b, uchar W1PIN )
{
  cli();
  W1_DDR |= 1<<W1PIN;
  DELAY( DELAY_US( 1 ));
  if( b )
    W1_DDR &= ~(1<<W1PIN);
  DELAY( DELAY_US( 15 - 1 ));
  if( (W1_IN & (1<<W1PIN)) == 0 )
    b = 0;
  DELAY( DELAY_US( 60 - 15 ));
  W1_DDR &= ~(1<<W1PIN);
  sei();
  return b;
}


uint w1_byte_wr( uchar b, uchar W1PIN )
{

  uchar i = 8, j;
  do{
    j = w1_bit_io( b & 1, W1PIN);
    b >>= 1;
    if( j )
      b |= 0x80;
  }while( --i );
  return b;
}


uint w1_byte_rd( uchar W1PIN )
{
  return w1_byte_wr( 0xFF, W1PIN);
}


uchar w1_rom_search( uchar diff, uchar idata *id, uchar W1PIN )
{
  uchar i, j, next_diff;
  bit b;

  if( w1_reset( W1PIN) )
    return PRESENCE_ERR;			// error, no device found
  w1_byte_wr( SEARCH_ROM, W1PIN);		// ROM search command
  next_diff = LAST_DEVICE;			// unchanged on last device
  i = 8 * 8;					// 8 bytes
  do{
    j = 8;					// 8 bits
    do{
      b = w1_bit_io( 1 , W1PIN);		// read bit
      if( w1_bit_io( 1 , W1PIN) ){		// read complement bit
	if( b )					// 11
	  return DATA_ERR;			// data error
      }else{
	if( !b ){				// 00 = 2 devices
	  if( diff > i ||
	    ((*id & 1) && diff != i) ){
	    b = 1;				// now 1
	    next_diff = i;			// next pass 0
	  }
	}
      }
      w1_bit_io( b , W1PIN);     		// write bit
      *id >>= 1;
      if( b )					// store bit
	*id |= 0x80;
      i--;
    }while( --j );
    id++;					// next byte
  }while( i );
  return next_diff;				// to continue search
}


void w1_command( uchar command, uchar idata *id, uchar W1PIN  )
{
  uchar i;
  w1_reset( W1PIN);
  if( id ){
    w1_byte_wr( MATCH_ROM,  W1PIN);			// to a single device
    i = 8;
    do{
      w1_byte_wr( *id , W1PIN);
      id++;
    }while( --i );
  }else{
    w1_byte_wr( SKIP_ROM,  W1PIN );			// to all devices
  }
  w1_byte_wr( command,  W1PIN );
}
