/* Name: main.c
 * Project: USB-Mega-TM based on PowerSwitch
 * Author: Wjatscheslaw Stolajrski
 * Version: 0.1
 * Creation Date: 2006-12-12
 * Changed: 2006-12-12
 * Tabsize: 4
 * License: Proprietary, free under certain conditions. See Documentation.
 * This Revision: $Id: main.c 412 2006-12-12 15:38Z sw $
 */

#define F_CPU	12000000L

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
//#include <util/delay.h>

#include "usbdrv.h"
#include "oddebug.h"
#include "main.h"
#include "def.h"

#ifndef TEST_DRIVER_SIZE    /* define this to check out size of pure driver */

#define STATUS_IDLE	0
#define STATUS_MEAS	1

int	retcode	= 0;
uchar 	status	= STATUS_IDLE;
tempWithID_t	twid;

uchar PinX=0;

void setxusbport(uchar b)
{
    PORTD = (PORTD & USBMASK) | (b & ~USBMASK);
}

void setxusbddr(uchar b)
{
    DDRD = (DDRD & USBMASK) | (b & ~USBMASK);
}

/* get count of temp. sensors */
int get_cnt(uchar W1Pin)
{
    uchar id[8], diff;
    
    int temp_cnt=0;

    for( diff = SEARCH_FIRST; diff != LAST_DEVICE; ){
	diff = w1_rom_search( diff, id ,W1Pin);

	if( diff == PRESENCE_ERR )
	    return -2;					// No Sensor found
	if( diff == DATA_ERR )
	    return -3;					// Bus Error
	if( id[0] == 0x28 || id[0] == 0x10 ){		// temperature sensor
	    temp_cnt++;
	}
    }
    return temp_cnt;
}

char start_meas( W1Pin )
{
    if( W1_IN & 1<< W1Pin ){
	w1_command( CONVERT_T, NULL, W1Pin);
	W1_OUT |= 1<< W1Pin;
	W1_DDR |= 1<< W1Pin;			// parasite power on
	return 0;
    }else{
	return -1;				// Short Circuit !
    }
}

int read_meas( uchar W1Pin )
{
    uchar  diff;
    int temp;

    twid.count = 0;

    for( diff = SEARCH_FIRST; diff != LAST_DEVICE; ){
	diff = w1_rom_search( diff, twid.tmp[(uchar) twid.count].id, W1Pin );

	if( diff == PRESENCE_ERR ){		    // No Sensor found
	    return -2;
	}
	if( diff == DATA_ERR ){			    // Bus Error
	    return -3;
	}
	if( twid.tmp[(uchar) twid.count].id[0] == 0x28 
        || twid.tmp[(uchar) twid.count].id[0] == 0x10 ){       // temperature sensor
	    w1_byte_wr( READ, W1Pin);                       // read command
	    temp = w1_byte_rd(W1Pin);                      // low byte
	    temp |= (uint)w1_byte_rd(W1Pin) << 8;          // high byte
//	    temp = 0xffce;			      // test for -25 C
	    if( twid.tmp[(uchar) twid.count].id[0] == 0x10 ){ // 9 -> 12 bit
		temp <<= 3;
		temp >>= 3;			      // vorzeichenrichtig erweitern
	    }
	    twid.tmp[(uchar) twid.count].Tt = temp;   //
	    twid.count++;
	    if(twid.count >= maxCountOfSensors)
		return twid.count;
	}
    }
    return 0;
}

void usbReset()
{
uchar   i, j;
    DDRD |= USBMASK;						/* USB pins outputs (-> USB reset) */
    j = 0;
    while(--j){			/* USB Reset by device only required on Watchdog Reset */
        i = 0;
        while(--i);		/* delay >10ms for USB reset */
    }
    DDRD = DDRD & (~USBMASK);    /* 0000 0101 bin: remove USB reset condition */
}

uchar   usbFunctionSetup(uchar data[8])
{
static uchar    replyBuf[8];

/* von Powerswitch
	data[0]
	data[1] - command
	data[2] - Pin
*/
    usbMsgPtr = replyBuf;

    if( (data[1] == UMTM_GETT) || (data[1] == UMTM_DEV_ENUM) ){
	usbMsgPtr = (void *) (& twid);
        PinX=data[2];
	return 1 + twid.count*10;
//        return 0xff;				// return 0xff + in header file param .... - for usbFunctionRead
    }
    if(data[1] == UMTM_STARTMEAS){
        //idx=data[2];
        PinX=data[2];
	second = 0;
	status = STATUS_MEAS;
    }
/*    if(data[1] == UMTM_DEV_ENUM){
    }
*/
    if(data[1] == UMTM_DEV_CNT){
        //idx=data[2]
	replyBuf[0] = twid.count;
        PinX=data[2];
	return 1;
    }
    return 0;
}

int main(void)
{
    
//    wdt_enable(WDTO_1S);
    twid.count = 0;
    init_timer();
    
//    PORTA = 0x0;
//    DDRA = 0x0;
    PORTB = 0x0;						/* activate pull-ups */
    DDRB = 0x0;							/* 1 for output, 0 for input */
    PORTC = 0x0;						/* activate pull-ups */
    DDRC = 0x0;							/* 1 for output, 0 for input */
    PORTD = 0x0;						/* no pull-ups on USB pins. */
    usbReset();
    odDebugInit();
    /*twid.count = */get_cnt(0);			/* get count of temp. sensors */
    get_cnt(1);
    get_cnt(2);
    usbInit();
    sei();
    for(;;){    /* main event loop */
//        wdt_reset();
        usbPoll();
	if(status == STATUS_MEAS){
	    if(second == 1){
		start_meas(PinX);
		second = 2;
	    }
	    if(second >= 3){
		retcode = read_meas(PinX);
//		second = 0;
		status = STATUS_IDLE;
//		usbReset();
	    }
	}
    }
    return 0;
}

#else   /* TEST_DRIVER_SIZE */

/* This is the minimum do-nothing function to determine driver size. The
 * resulting binary will consist of the C startup code, space for interrupt
 * vectors and our minimal initialization. The C startup code and vectors
 * (together ca. 70 bytes of flash) can not directly be considered a part
 * of this driver. The driver is therefore ca. 70 bytes smaller than the
 * resulting binary due to this overhead. The driver also contains strings
 * of arbitrary length. You can save another ca. 50 bytes if you don't
 * include a textual product and vendor description.
 */
uchar   usbFunctionSetup(uchar data[8])
{
    return 0;
}

int main(void)
{
#ifdef PORTD
    PORTD = 0;
    DDRD = ~(1 << 2);   /* all outputs except PD2 = INT0 */
#endif
    PORTB = 0;          /* no pullups on USB pins */
    DDRB = ~USBMASK;    /* all outputs except USB data */
    usbInit();
    sei();
    for(;;){    /* main event loop */
        usbPoll();
    }
    return 0;
}

#endif  /* TEST_DRIVER_SIZE */
